"""hotfix_ncbi_checkaccession

The tools at NCBI are constantly evolving. The functionality used to check the availability of SRA accession codes 
has been updated regularly.

Therefore, importing SRS experiments as links to NCBI/SRA accession codes, and submitting jobs for these links
is no longer possible. This script replaces the check function with an updated version that restores the functionality.

Checking the NCBI SRA accession codes started to fail on August 20, 2022.
The reason is that the NCBI service discontinued the use of the names.cgi page.

Checking the NCBI SRA accession codes started to fail on July 1, 2023.
The reason is that the NCBI service removed the names.cgi page.

"""

#//$autorun;event=PostCreateMainWin;

import bns
	
# make sure the wgMLST_Client plugin is installed
try:
	import wgMLST_Client.importSRSLinks.DownloadSites as DS
except ImportError:
	bns.Stop()

from time import sleep, time
import xml.etree.cElementTree as ET

def ReplaceCheckLink(downloader, NewCheckLink):
	"Replacing the CheckLink function for any downloader type"

	# replace the CheckLink function
	downloader.CheckLink = NewCheckLink
	
	# set explicitly for downloaders
	idx = DS.downloaderList.index(downloader)
	DS.downloaderList[idx] = downloader
	DS.downloaders[downloader.ID] = downloader
	
	# set explicitly for import wizard downloaders
	import wgMLST_Client.importSRSLinks.importwizard
	wgMLST_Client.importSRSLinks.importwizard.downloaderList[idx] = downloader
	wgMLST_Client.importSRSLinks.importwizard.downloaders[downloader.ID] = downloader
	
	# set explicitly for job submission checks as well
	import wgMLST_Client.WgmlstLaunchSingleEntryJobsDlg
	wgMLST_Client.WgmlstLaunchSingleEntryJobsDlg.downloaders[downloader.ID] = downloader


def NCBICheckLink(accCode):
	"""Check SRA accession code using eutils API
	"""
	def DelayedUrlGet(url, delay):
		# there is a limit of 3 requests per second for eutils API -> delay between requests
		start = time()	
		response = bns.Util.UrlTools.urlget(url)
		responseTime = time() - start
		if responseTime < delay:
			sleep(delay - responseTime)
		return response
	
	# use eutils to check the SRA accession code
	url = 'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=SRA&term={0}[Accession]%20AND%20cluster_public[prop]'.format(accCode)
	response = DelayedUrlGet(url, delay=0.5)
	responseXML = ET.fromstring(response)
	count = responseXML.findtext('Count')
	if count == '1':
		return True
	elif count == '0':
		raise RuntimeError("Accession {0} was not found.".format(accCode))
	elif count.isdigit() and int(count) > 1:
		raise RuntimeError("Accession {0} is not unique.".format(accCode))
	else:
		# count is non-number or negative...
		raise RuntimeError(response)


if __name__ == '__main__':
	# We only want to replace the accession code check if it is broken,
	# so we can distribute a fix in a patch/update of BioNumerics.
	# Therefore we first try the built-in link checker and see if it produces a specific error
	ncbi = DS.downloaders[DS.NCBI_Downloader.ID]
	try:
		ncbi.CheckLink('SRR1508820') # we know for sure this accession exists
	except Exception as e:
		errorMsg = '{0}'.format(e).strip()
		if errorMsg == '||||||||400|Unsupported protocol':
			# NCBI checks the protocol query key => replace the check function
			ReplaceCheckLink(ncbi, NCBICheckLink)
		elif errorMsg == 'HTTP Error 403: Forbidden':
			# NCBI moves from http to https
			ReplaceCheckLink(ncbi, NCBICheckLink)
		elif errorMsg == 'HTTP Error 404: Not Found':
			# NCBI removed the /names.cgi page -> resoure not found
				ReplaceCheckLink(ncbi, NCBICheckLink)
		elif errorMsg == 'HTTP Error 406: Not Acceptable':
			# NCBI suppressed the names.cgi functionality
			ReplaceCheckLink(ncbi, NCBICheckLink)
		elif errorMsg == 'Obsolete software. See https://github.com/ncbi/sra-tools/wiki':
			# NCBI update to HTTPS
			# on 2016-12-06: NCBI discontinues 'version=1.1'
			ReplaceCheckLink(ncbi, NCBICheckLink)
		else:
			# any other error -> leave the functionality as-is
			if __bnsdebug__:
				print(e)
